# Enable -Verbose option
[CmdletBinding()]
	
# Disable parameter
# Convenience option so you can debug this script or disable it in 
# your build definition without having to remove it from
# the 'Post-build script path' build process parameter.
param([switch]$Disable, [string]$tfsUri="", [string]$workingSubDir="")
if ($PSBoundParameters.ContainsKey('Disable'))
{
	Write-Verbose "Script disabled; no actions will be taken on the files."
}

Write-Verbose "tfsUri: $tfsUri"
Write-Verbose "workingSubDir: $workingSubDir"

$sourceDir = $Env:TF_BUILD_SOURCESDIRECTORY, $sourceSubDir
Write-Verbose "sourceDir: $sourceDir"

# where is the config file for the composed build
$configFilePath = [System.IO.Path]::Combine($sourceDir, "composed-build-config.xml")
Write-Verbose "configFilePath: $configFilePath"

# path for the composed build DLL
$composedBuildBinaryPath = [System.IO.Path]::Combine($sourceDir, "BuildBinaries", "Benday.ComposedBuild.Core.dll")
Write-Verbose "composedBuildBinaryPath: $composedBuildBinaryPath"

# working directory path where the referenced build outputs 
# will be downloaded to.  
$workingDir = [System.IO.Path]::Combine($sourceDir, $workingSubDir)
Write-Verbose "workingDir: $workingDir"

if (![System.IO.Directory]::Exists($workingDir))
{
	[System.IO.Directory]::CreateDirectory($workingDir)
}

# load the composed build assembly from the DLL
[System.Reflection.Assembly]::LoadFrom($composedBuildBinaryPath)

# create an instance of the ComposedBuildManager
$downloader = new-object Benday.ComposedBuild.Core.ComposedBuildManager

# read the contents of the config file
$configFileContents = [System.IO.File]::ReadAllText($configFilePath)
Write-Verbose "config contents: $configFileContents"

# initialize the ComposedBuildManager
$downloader.Initialize($configFileContents, $tfsUri, $workingDir)

Write-Verbose "Starting download..."
$downloader.Download()
Write-Verbose "Download completed."