using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Benday.ComposedBuild.Core
{
    public class BuildOutput
    {
        public BuildOutput(XElement fromValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");

            Description = fromValue.AttributeValue("description");
            TeamProject = fromValue.AttributeValue("team-project");
            BuildName = fromValue.AttributeValue("build-name");
            BuildNumber = fromValue.AttributeValue("build-number");
            DropDirectorySubfolder = fromValue.AttributeValue("drop-directory-subfolder");
            ToFolder = fromValue.AttributeValue("to-folder");

            HasBeenInitializedForDownload = false;
        }

        public string Description { get; set; }
        public string TeamProject { get; set; }
        public string BuildName { get; set; }
        public string BuildNumber { get; set; }
        public string DropDirectorySubfolder { get; set; }
        public string ToFolder { get; set; }

        public bool HasBeenInitializedForDownload { get; private set; }
        public string DropDirectoryUncPath { get; set; }
        public string LocalWorkingFolder { get; set; }

        public void InitializeForDownload(string dropDirectoryUncPath, string localWorkingFolder)
        {
            if (String.IsNullOrEmpty(dropDirectoryUncPath))
                throw new ArgumentException("dropDirectoryUncPath is null or empty.", "dropDirectoryUncPath");
            if (String.IsNullOrEmpty(localWorkingFolder))
                throw new ArgumentException("localWorkingFolder is null or empty.", "localWorkingFolder");

            DropDirectoryUncPath = dropDirectoryUncPath;
            LocalWorkingFolder = localWorkingFolder;

            HasBeenInitializedForDownload = true;
        }

        public override string ToString()
        {
            string template = "{0} from {1} build number {2}";

            return String.Format(template, Description, TeamProject, BuildNumber);
        }

        private void AssertHasBeenInitialized()
        {
            if (HasBeenInitializedForDownload == false)
            {
                throw new InvalidOperationException("InitializeForDownload() has not been called.");
            }
        }

        public string SourcePath
        {
            get
            {
                AssertHasBeenInitialized();

                return Path.Combine(DropDirectoryUncPath, DropDirectorySubfolder);
            }
        }

        public string TargetPath
        {
            get
            {
                AssertHasBeenInitialized();

                return Path.Combine(LocalWorkingFolder, ToFolder);
            }
        }
    }
}
