using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.ComposedBuild.Core;
using System.Collections.Generic;
using System.IO;
using System.Xml.Linq;

namespace Benday.ComposedBuild.Tests
{
    [TestClass]
    public class BuildOutputDownloadFixture
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
        }

        private BuildOutput _SystemUnderTest;
        public BuildOutput SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new BuildOutput(
                        GetXElement()
                        );
                }

                return _SystemUnderTest;
            }
        }
        private System.Xml.Linq.XElement GetXElement()
        {
            return XElement.Parse(Resource1.SampleBuildOutputDownload);
        }

        [TestMethod]
        public void SutIsNotNull()
        {
            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void DescriptionIsPopulated()
        {
            Assert.AreEqual<string>("wpf app", SystemUnderTest.Description, "Value is wrong.");
        }

        [TestMethod]
        public void TeamProjectIsPopulated()
        {
            Assert.AreEqual<string>("team-project-2", SystemUnderTest.TeamProject, "Value is wrong.");
        }

        [TestMethod]
        public void BuildNameIsPopulated()
        {
            Assert.AreEqual<string>("WpfApplication1-compile", SystemUnderTest.BuildName, "Value is wrong.");
        }

        [TestMethod]
        public void BuildNumberIsPopulated()
        {
            Assert.AreEqual<string>("WpfApplication1-compile_20131220.2",
                SystemUnderTest.BuildNumber, "Value is wrong.");
        }

        [TestMethod]
        public void ToStringReturnsExpectedValue()
        {
            var expected = "wpf app from team-project-2 build number WpfApplication1-compile_20131220.2";

            string actual = SystemUnderTest.ToString();

            Assert.AreEqual<string>(expected, actual, "ToString() returned wrong value.");
        }

        [TestMethod]
        public void DropDirectorySubfolderIsPopulated()
        {
            Assert.AreEqual<string>("subfolder",
                SystemUnderTest.DropDirectorySubfolder, "Value is wrong.");
        }

        [TestMethod]
        public void ToFolderIsPopulated()
        {
            Assert.AreEqual<string>("wpfapp",
                SystemUnderTest.ToFolder, "Value is wrong.");
        }

        [TestMethod]
        public void HasBeenInitializedForDownloadIsFalseWhenInitializeHasNotBeenCalled()
        {
            Assert.IsFalse(SystemUnderTest.HasBeenInitializedForDownload);
        }

        private void CallInitializeForDownload()
        {
            SystemUnderTest.InitializeForDownload(
                            @"\\tfsbuild2013\",
                            @"c:\temp\BuildOutputDownloadManagerFixture\");
        }

        [TestMethod]
        public void HasBeenInitializedForDownloadIsTrueWhenInitializeHasNotBeenCalled()
        {
            CallInitializeForDownload();

            Assert.IsTrue(SystemUnderTest.HasBeenInitializedForDownload);
        }

        [TestMethod]
        public void LocalWorkingFolderIsPopulatedWhenInitializeForDownloadIsCalled()
        {
            CallInitializeForDownload();

            Assert.IsNotNull(SystemUnderTest.LocalWorkingFolder);
            Assert.AreNotEqual<string>(String.Empty, 
                SystemUnderTest.LocalWorkingFolder, 
                "LocalWorkingFolder should not be empty.");
        }

        [TestMethod]
        public void LocalWorkingFolderIsNullBeforeInitializeForDownloadIsCalled()
        {
            Assert.IsNull(SystemUnderTest.LocalWorkingFolder);
        }

        [TestMethod]
        public void DropDirectoryUncPathIsPopulatedWhenInitializeForDownloadIsCalled()
        {
            CallInitializeForDownload();

            Assert.IsNotNull(SystemUnderTest.LocalWorkingFolder);
            Assert.AreNotEqual<string>(String.Empty,
                SystemUnderTest.DropDirectoryUncPath,
                "DropDirectoryUncPath should not be empty.");
        }

        [TestMethod]
        public void DropDirectoryUncPathIsNullBeforeInitializeForDownloadIsCalled()
        {
            Assert.IsNull(SystemUnderTest.DropDirectoryUncPath);
        }

        [TestMethod]
        public void SourcePathIsPopulatedWhenInitializeForDownloadIsCalled()
        {
            CallInitializeForDownload();

            string expected = @"\\tfsbuild2013\subfolder";
            string actual = SystemUnderTest.SourcePath;

            Assert.AreEqual<string>(expected, actual, "SourcePath is wrong.");
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SourcePathThrowsInvalidOperationExceptionWhenInitializeForDownloadIsCalled()
        {
            Assert.IsFalse(SystemUnderTest.HasBeenInitializedForDownload);

            string actual = SystemUnderTest.SourcePath;

            Console.WriteLine(actual);
        }

        [TestMethod]
        public void TargetPathIsPopulatedWhenInitializeForDownloadIsCalled()
        {
            CallInitializeForDownload();

            string expected = @"c:\temp\BuildOutputDownloadManagerFixture\wpfapp";
            string actual = SystemUnderTest.TargetPath;

            Assert.AreEqual<string>(expected, actual, "SourcePath is wrong.");
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TargetPathThrowsInvalidOperationExceptionWhenInitializeForDownloadIsCalled()
        {
            Assert.IsFalse(SystemUnderTest.HasBeenInitializedForDownload);

            string actual = SystemUnderTest.TargetPath;

            Console.WriteLine(actual);
        }
    }
}
