using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.ComposedBuild.Core;
using System.Collections.Generic;

namespace Benday.ComposedBuild.Tests
{
    [TestClass]
    public class ListTfsBuildDefinitionsFixture : UnitTestBase
    {
        private TfsSession _SystemUnderTest;
        public TfsSession SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = 
                        new TfsSession(TfsCollectionUrl);
                }

                return _SystemUnderTest;
            }
        }

        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
        }

        [TestMethod]
        public void ListAllBuildDefinitions()
        {
            List<string> buildDefs = SystemUnderTest.GetBuildDefinitions(TeamProjectName);

            Assert.IsNotNull(buildDefs);
            Assert.AreNotEqual<int>(0, buildDefs.Count, "No build definitions.");

            buildDefs.ForEach(x => Console.WriteLine(x));
        }

        [TestMethod]
        public void ListAllBuildsForDefinition()
        {
            List<string> builds = SystemUnderTest.GetBuildsForDefinition(
                TeamProjectName, "BendayWebTestSample-Server-Compile");

            Assert.IsNotNull(builds);
            Assert.AreNotEqual<int>(0, builds.Count, "No build definitions.");

            builds.ForEach(x => Console.WriteLine(x));
        }


        [TestMethod]
        public void GetBuildInfoForBuild()
        {
            string expectedBuildNumber = "BendayWebTestSample-Server-Compile_20131219.3";

            string expectedBuildDefinitionName = "BendayWebTestSample-Server-Compile";

            var actual = SystemUnderTest.GetBuildInfo(
                TeamProjectName, expectedBuildDefinitionName,
                expectedBuildNumber);

            Assert.IsNotNull(actual, "Build info was null.");

            Assert.AreEqual<string>(
                expectedBuildNumber, actual.BuildNumber, "Build number was wrong.");
            Assert.AreEqual<string>(
                expectedBuildDefinitionName, actual.BuildName, "Build name was wrong.");

            Assert.IsNotNull(actual.DropLocation, "DropLocation should not be null.");
            Console.WriteLine(actual.DropLocation);
            Assert.AreNotEqual<string>(String.Empty, actual.DropLocation, "DropLocation should not be empty.");
        }
    }
}
